@@ -1,4 +1,9 @@
 from yafowil.base import factory
+from yafowil.compound import (
+    compound_extractor,
+    compound_renderer,
+    hybrid_extractor,
+)
 from yafowil.utils import (
     cssclasses,
     css_managed_props,
@@ -108,11 +113,16 @@ def td_renderer(widget, data):
         'colspan': widget.attrs.get('colspan'),
         'rowspan': widget.attrs.get('rowspan'),
     }
-    return data.tag('td', data.rendered, **attrs)
+    if len(widget):
+        rendered = compound_renderer(widget, data)
+    else:
+        rendered = data.rendered
+    return data.tag('td', rendered, **attrs)
 
 
 factory.register(
     'td',
+    extractors=[hybrid_extractor],
     edit_renderers=[td_renderer],
     display_renderers=[td_renderer])
 
